/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.database.entity;

import com.sos.scheduler.engine.kernel.database.DatabaseSubsystem;
import java.util.Date;
import javax.persistence.*;
import static com.google.common.base.Strings.*;
import static javax.persistence.TemporalType.*;


@Entity
@Table(name="SCHEDULER_HISTORY")
public class TaskHistoryEntity {
    public static final String schedulerDummyJobPath = "(Spooler)";
    
    @Id private int id;
    @Column(name="spooler_id") private String schedulerId;
    @Column(name="cluster_member_id") private String clusterMemberId;
    @Column(name="job_name") private String jobPath;
    @Column(name="start_time") @Temporal(TIMESTAMP) private Date startTime;
    @Column(name="end_time") @Temporal(TIMESTAMP) private Date endTime;
    @Column(name="cause") private String cause;
    @Column(name="steps") private Integer steps;
    @Column(name="error_code") private String errorCode;
    @Column(name="error_text") private String errorText;


    public int getId() {
        return id;
    }


    public void setId(int id) {
        this.id = id;
    }


    public String getSchedulerId() {
        return schedulerId.equals(DatabaseSubsystem.emptyIdinDatabase)? "" : schedulerId;
    }


    public void setSchedulerId(String schedulerID) {
        this.schedulerId = DatabaseSubsystem.idForDatabase(schedulerID);
    }


    public String getClusterMemberId() {
        return nullToEmpty(clusterMemberId);
    }


    public void setClusterMemberId(String clusterMemberId) {
        this.clusterMemberId = emptyToNull(clusterMemberId);
    }


    public String getJobPath() {
        return jobPath.equals(schedulerDummyJobPath)? "" : "/" + jobPath;
    }


    public void setJobPath(String p) {
        this.jobPath = p.isEmpty()? schedulerDummyJobPath : p.replaceFirst("^/", "");
    }


    @Override public String toString() {
        return "schedulerId=" + getSchedulerId() +
                " clusterMemberId=" + getClusterMemberId() +
                " jobName=" + getJobPath();
    }


    public String getCause() {
        return cause;
    }


    public Integer getSteps() {
        return steps;
    }


    public String getErrorCode() {
        return errorCode;
    }


    public String getErrorText() {
        return errorText;
    }


    public Date getStartTime() {
        return startTime;
    }


    public Date getEndTime() {
        return endTime;
    }
}
