/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.test;

import com.sos.scheduler.engine.kernel.Scheduler;
import com.sos.scheduler.engine.kernel.event.EventSubscriber;
import com.sos.scheduler.engine.kernel.main.SchedulerController;
import com.sos.scheduler.engine.kernel.main.SchedulerThread;
import com.sos.scheduler.engine.kernel.util.Time;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import org.apache.log4j.Logger;
import org.junit.After;


public abstract class SchedulerTest {
    public static final Time shortTimeout = Time.of(10);
    private static final Logger logger = Logger.getLogger(SchedulerTest.class);

    protected final SchedulerController schedulerController;
    protected final Iterable<String> resourceNames;
    protected Scheduler scheduler = null;
    private final Environment env;


    public SchedulerTest(Iterable<String> resourceNames) {
        schedulerController = new SchedulerThread();
        this.resourceNames = resourceNames;
        this.env = new Environment(this, resourceNames);
    }


    public SchedulerTest() {
        this(new ArrayList<String>());
    }


    public final void strictSubscribeEvents() {
        strictSubscribeEvents(EventSubscriber.empty);
    }


    public final void strictSubscribeEvents(EventSubscriber s) {
        schedulerController.subscribeEvents(new StrictEventSubscriber(s));
    }


    public final void runScheduler(Time timeout, String... args) {
        startScheduler(args);
        waitUntilSchedulerIsRunning();
        waitForTermination(timeout);
    }


//    public void runScheduler(String... args) {
//        runScheduler(Time.eternal, args);
//    }


    public final void startScheduler(String... args) {
        ArrayList<String> allArgs = new ArrayList<String>();
        allArgs.addAll(Arrays.asList(env.standardArgs()));
        allArgs.addAll(Arrays.asList(args));
        schedulerController.startScheduler(allArgs.toArray(new String[0]));
    }


    public final void waitUntilSchedulerIsRunning() {
        scheduler = schedulerController.waitUntilSchedulerIsRunning();
    }


    public final Scheduler getScheduler() {
        if (scheduler == null)
            waitUntilSchedulerIsRunning();
        assert scheduler != null;
        return scheduler;
    }


    public final void waitForTermination(Time timeout) {
        schedulerController.waitForTermination(timeout);
    }


    @After public final void terminateAndCleanUp() throws Throwable {
        try {
            schedulerController.terminateAndWait();
        }
        catch (Throwable x) {
            logger.error(SchedulerTest.class.getName() + " @After: " + x, x);
            throw x;
        }
        finally {
            env.cleanUp();
        }
    }


    public final File getDirectory() {
        return env.getDirectory();
    }
}
