/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.scheduler.job;

import java.io.File;
import java.util.Vector;

import sos.scheduler.file.JobSchedulerFileOperationBase;
import sos.util.SOSFile;

import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.i18n.annotation.I18NMessage;
import com.sos.i18n.annotation.I18NMessages;
import com.sos.i18n.annotation.I18NResourceBundle;

/**
 * @author andreas.pueschel@sos-berlin.com
 *
 * 
 * cleanup temporary files created by other jobs
 * Paramters:
 * file_path Path for the files to remove
 * file_specification Regular expression as file specification
 * file_age Minimum file-age either milliseconds or hh:mm[:ss]
 * $Id: JobSchedulerCleanupFiles.java 15334 2011-10-17 07:44:50Z oh $
 */
@I18NResourceBundle(baseName = "com.sos.scheduler.messages", defaultLocale = "en")
public class JobSchedulerCleanupFiles extends JobSchedulerFileOperationBase {
	private final String		conSVNVersion	= "$Id: JobSchedulerCleanupFiles.java 15334 2011-10-17 07:44:50Z oh $";
	@SuppressWarnings("hiding")
	private final static String	conClassName	= "JobSchedulerCleanupFiles";

	public boolean spooler_init() {
		return super.spooler_init();
	}

	public boolean spooler_process() {

		initialize(conSVNVersion);

		try {
			String[] filePaths = filePath.split(";");
			if (isNull(fileSpec)) {
				fileSpec = "^(sos.*)";
			}
			String[] fileSpecs = fileSpec.split(";");
			boolean flgPathAndSpecHasSameNumberOfItems = (filePaths.length == fileSpecs.length);
			fileSpec = fileSpecs[0];
			for (int i = 0; i < filePaths.length; i++) {
				int counter = 0;
				filePath = filePaths[i];
				if (filePath.trim().equalsIgnoreCase(conPropertyJAVA_IO_TMPDIR)) {
					filePath = System.getProperty(conPropertyJAVA_IO_TMPDIR);
				}
				if (flgPathAndSpecHasSameNumberOfItems == true) {
					fileSpec = fileSpecs[i];
				}

				logger.debug(Messages.getMsg(JSJ_I_0014, filePath));
				Vector<File> filelist = SOSFile.getFolderlist(filePath, fileSpec, 0);
				if (filelist.size() == 0) {
					logger.info(Messages.getMsg(JSJ_I_0013));
				}

				if (warningFileLimit > 0 && filelist.size() >= warningFileLimit) {
					logger.error(Messages.getMsg(JSJ_E_0016, filelist.size(), filePath, warningFileLimit, conParameterWARNING_FILE_LIMIT));
				}

				for (File tempFile : filelist) {
					long interval = System.currentTimeMillis() - tempFile.lastModified();
					if (interval > lngFileAge) {
						counter += SOSFile.deleteFile(tempFile);
						logger.info(Messages.getMsg(JSJ_I_0012, tempFile.getAbsolutePath()));
					}
				} // for
				if (counter > 0) {
					String strT = (filePath.equals(System.getProperty(conPropertyJAVA_IO_TMPDIR)) ? " temporary" : "");
					logger.info(Messages.getMsg(JSJ_I_0015, counter, strT));
				}
			} // for

		}
		catch (Exception e) {
			throw new JobSchedulerException(Messages.getMsg(Messages.getMsg(JSJ_F_0010), conClassName, e.getLocalizedMessage()), e);
		}

		// return false;
		return signalSuccess();
	}
	@I18NMessages(value = { @I18NMessage("File deleted: %1$s"), //
			@I18NMessage(value = "File deleted: %1$s", //
			locale = "en_UK", //
			explanation = "File deleted: %1$s" //
			), //
			@I18NMessage(value = "Datei '%1$s' wurde gelscht.", //
			locale = "de", //
			explanation = "File deleted: %1$s" //
			), //
			@I18NMessage(value = "File deleted: %1$s", locale = "es", //
			explanation = "File deleted: %1$s" //
			), //
			@I18NMessage(value = "File deleted: %1$s", locale = "fr", //
			explanation = "File deleted: %1$s" //
			), //
			@I18NMessage(value = "File deleted: %1$s", locale = "it", //
			explanation = "File deleted: %1$s" //
			) //
	}, msgnum = "JSJ_I_0012", msgurl = "JSJ_I_0012")
/*!
 * \var JSJ_I_0012
 * \brief "File deleted: %1$s"
 */
	public static final String	JSJ_I_0012	= "JSJ_I_0012";

	@I18NMessages(value = { @I18NMessage(".. No files matched."), //
			@I18NMessage(value = ".. No files matched.", //
			locale = "en_UK", //
			explanation = ".. No files matched." //
			), //
			@I18NMessage(value = ".. No files matched.", //
			locale = "de", //
			explanation = ".. No files matched." //
			), //
			@I18NMessage(value = ".. No files matched.", locale = "es", //
			explanation = ".. No files matched." //
			), //
			@I18NMessage(value = ".. No files matched.", locale = "fr", //
			explanation = ".. No files matched." //
			), //
			@I18NMessage(value = ".. No files matched.", locale = "it", //
			explanation = ".. No files matched." //
			) //
	}, msgnum = "JSJ_I_0013", msgurl = "JSJ_I_0013")
/*!
 * \var JSJ_I_0013
 * \brief .. No files matched.
 */
	public static final String	JSJ_I_0013	= "JSJ_I_0013";

	@I18NMessages(value = { @I18NMessage("Looking for files in: %1$s"), //
			@I18NMessage(value = "Looking for files in: %1$s", //
			locale = "en_UK", //
			explanation = "Looking for files in: %1$s" //
			), //
			@I18NMessage(value = "Dateisuche startet fr Verzeichnis: %1$s", //
			locale = "de", //
			explanation = "Looking for files in: %1$s" //
			), //
			@I18NMessage(value = "Looking for files in: %1$s", locale = "es", //
			explanation = "Looking for files in: %1$s" //
			), //
			@I18NMessage(value = "Looking for files in: %1$s", locale = "fr", //
			explanation = "Looking for files in: %1$s" //
			), //
			@I18NMessage(value = "Looking for files in: %1$s", locale = "it", //
			explanation = "Looking for files in: %1$s" //
			) //
	}, msgnum = "JSJ_I_0014", msgurl = "JSJ_I_0014")
/*!
 * \var JSJ_I_0014
 * \brief Looking for files in: %1$s
 */
	public static final String	JSJ_I_0014	= "JSJ_I_0014";

	@I18NMessages(value = { @I18NMessage("%1$d %2$s files deleted"), //
			@I18NMessage(value = "%1$d %2$s files deleted", //
			locale = "en_UK", //
			explanation = "%1$d %2$s files deleted" //
			), //
			@I18NMessage(value = "%1$d %2$s Dateien wurden gelscht.", //
			locale = "de", //
			explanation = "%1$d %2$s files deleted" //
			), //
			@I18NMessage(value = "%1$d %2$s files deleted", locale = "es", //
			explanation = "%1$d %2$s files deleted" //
			), //
			@I18NMessage(value = "%1$d %2$s files deleted", locale = "fr", //
			explanation = "%1$d %2$s files deleted" //
			), //
			@I18NMessage(value = "%1$d %2$s files deleted", locale = "it", //
			explanation = "%1$d %2$s files deleted" //
			) //
	}, msgnum = "JSJ_I_0015", msgurl = "JSJ_I_0015")
/*!
 * \var JSJ_I_0015
 * \brief %1$d %2$s files deleted
 */
	public static final String	JSJ_I_0015	= "JSJ_I_0015";

	@I18NMessages(value = { @I18NMessage("'%1$s' files were found in directory '%2$s'. That is more than specified with param '%4$s' = '%3$d'."), //
			@I18NMessage(value = "'%1$s' files were found in directory '%2$s'. That is more than specified with param '%4$s' = '%3$d'.", //
			locale = "en_UK", //
			explanation = "'%1$s' files were found in directory '%2$s'. That is more than specified with param '%4$s' = '%3$d'." //
			), //
			@I18NMessage(value = "'%1$s' Dateien im Verzeichnis '%2$s' gefunden. Das sind mehr als im Parameter '%4$s' angegeben mit '%3$d'.", //
			locale = "de", //
			explanation = "'%1$s' files were found in directory '%2$s'. That is more than specified with param '%4$s' = '%3$d'." //
			), //
			@I18NMessage(value = "'%1$s' files were found in directory '%2$s'. That is more than specified with param '%4$s' = '%3$d'.", locale = "es", //
			explanation = "'%1$s' files were found in directory '%2$s'. That is more than specified with param '%4$s' = '%3$d'." //
			), //
			@I18NMessage(value = "'%1$s' files were found in directory '%2$s'. That is more than specified with param '%4$s' = '%3$d'.", locale = "fr", //
			explanation = "'%1$s' files were found in directory '%2$s'. That is more than specified with param '%4$s' = '%3$d'." //
			), //
			@I18NMessage(value = "'%1$s' files were found in directory '%2$s'. That is more than specified with param '%4$s' = '%3$d'.", locale = "it", //
			explanation = "'%1$s' files were found in directory '%2$s'. That is more than specified with param '%4$s' = '%3$d'." //
			) //
	}, msgnum = "JSJ_E_0016", msgurl = "JSJ_E_0016")
/*!
 * \var JSJ_E_0016
 * \brief '%1$s' files were found in directory '%2$s'. That is more than specified with param '%4$s' = '%3$d'.
 */
	public static final String	JSJ_E_0016	= "JSJ_E_0016";

}
