/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: file_logger.h 15047 2011-08-26 08:23:12Z jz $        Joacim Zschimmer, Zschimmer GmbH, http://www.zschimmer.com

#ifndef __SCHEDULER_FILE_LOGGER_H
#define __SCHEDULER_FILE_LOGGER_H

#include "../zschimmer/mutex.h"

namespace sos {
namespace scheduler {

//------------------------------------------------------------------------------------Stdout_reader

struct File_logger : Async_operation
{
    struct File_line_reader : Object
    {
                                File_line_reader            (const File_path&, const string& name);

        void                    close                       ();
        string                  read_lines                  ();
        string                  read_remainder              ();

        Fill_zero              _zero_;
        File_path              _path;
        size_t                 _read_length;
        string                 _name;
        bool                   _error;
    };


    struct File_logger_thread : Thread
    {
                                File_logger_thread          ( File_logger* );
                               ~File_logger_thread          ();

        int                     thread_main                 ();
        void                    terminate                   ();

      private:
        Fill_zero              _zero_;

        File_logger*           _file_logger;
        Async_manager          _async_manager;
        zschimmer::Event       _terminate_event;
    };

                                File_logger                 ( Has_log* );
                               ~File_logger                 ();

    void                        close                       ();
    void                        set_object_name             ( const string& text )                  { _for_object = text; }
    bool                        has_files                   () const                                { return !_file_line_reader_list.empty(); }
  //void                        set_remove_files            ( bool b )                              { _remove_files = b; }
    void                        add_file                    ( const File_path&, const string& prefix );
    void                        start                       ();
    void                        start_thread                ();
    void                        finish                      ();
    bool                        flush                       ();
    bool                        flush_lines                 ();

  protected:
    // Async_operation:
    string                      async_state_text_           () const;
    bool                        async_continue_             ( Continue_flags );
    bool                        async_finished_             () const                                { return false; }


  private:
    bool                        log_lines                   ( const string& );


    typedef list< ptr<File_line_reader> >  File_line_reader_list;

    Fill_zero                  _zero_;
    Mutex                      _mutex;
    Has_log*                   _log;
    File_line_reader_list      _file_line_reader_list;
    ptr<File_logger_thread>    _thread;
  //bool                       _remove_files;
    string                     _for_object;
};

//-------------------------------------------------------------------------------------------------

} //namespace scheduler
} //namespace sos

#endif
