/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#include "spooler.h"
#include "log_cache_Request_cache.h"
#include "log_cache_Request.h"

namespace sos {
namespace scheduler {
namespace log {
namespace cache {

struct Request_cache_impl : Request_cache {
    private: typedef stdext::hash_map<Prefix_log*,ptr<Request> > Map;

    private: Fill_zero  _zero_;
    private: Map        _map;
    private: int64      _next_number;

    public: Request_cache_impl() : _zero_(this+1) {}

    public: ~Request_cache_impl() {
        Z_DEBUG_ONLY(assert(_map.empty()));
    }

    public: void remove(Prefix_log* log) {
        Map::iterator it = _map.find(log);
        if (it != _map.end())
            _map.erase(it);
    }

    public: void cache(Prefix_log* log) {
        cache(request(log));
    }

    public: ptr<Request> request(Prefix_log* log) {
        ptr<Request> result = cached_request(log);
        if (!result) {
            if (becomes_full())
                remove_oldest();
            result = Request::of(this, log, _next_number++);
            if (result)
                cache(result);
        }
        return result;
    }

    public: void cache(Request* request) {
        Map::iterator it = _map.find(request->log());
        if (it == _map.end()) {
            if (is_full())
                remove_oldest();
            _map[request->log()] = request;
            assert(_map.size() <= max_open_log_files);
        }
    }

    private: void remove_oldest() {
        _map.erase(oldest());
    }

    private: ptr<Request> cached_request(Prefix_log* log) {
        Map::iterator it = _map.find(log);
        return it == _map.end()? NULL : it->second;
    }

    private: Map::iterator oldest() {
        Map::iterator result = _map.end();
        Z_FOR_EACH(Map, _map, it)
            if (result == _map.end()  ||  it->second->number() < result->second->number())
                result = it;
        return result;
    }

    private: bool is_full() { return _map.size() >= max_open_log_files; }
    private: bool becomes_full() { return _map.size() + 1 >= max_open_log_files; }
};


ptr<Request_cache> Request_cache::new_instance() {
    ptr<Request_cache_impl> result = Z_NEW(Request_cache_impl); 
    return +result;
}

}}}}
