/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: xml_client_connection.h 13198 2007-12-06 14:13:38Z jz $        Joacim Zschimmer, Zschimmer GmbH, http://www.zschimmer.com

namespace sos {
namespace scheduler {

//----------------------------------------------------------------------------Xml_client_connection

struct Xml_client_connection : Async_operation, Scheduler_object
{
    enum State
    {
        s_not_connected,
        s_connecting,
        s_connected,
        s_sending,
        s_waiting,
        s_receiving,
        s_closed
    };

    static string               state_name                  ( State );


                                Xml_client_connection       ( Spooler*, const Host_and_port& );
                               ~Xml_client_connection       ();

    void                        close                       ();
    virtual string              obj_name                    () const;

    void                    set_wait_for_connection         ( int seconds )                         { _wait_for_connection = seconds; }
    State                       state                       () const                                { return _state; }

    void                        connect                     ();
    bool                        is_send_possible            ();
    void                        send                        ( const string& );
    xml::Document_ptr           fetch_received_dom_document ();                                     // NULL, wenn noch nichts empfangen


  protected:
    string                      async_state_text_           () const;
    bool                        async_continue_             ( Continue_flags );
    bool                        async_finished_             () const                                { return _state == s_not_connected 
                                                                                                          || _state == s_connected
                                                                                                          || _state == s_closed; }
    bool                        async_signaled_             ()                                      { return _socket_operation && _socket_operation->async_signaled(); }

  private:
    Fill_zero                  _zero_;
    State                      _state;
    Host_and_port              _host_and_port;
    int                        _wait_for_connection;
    ptr<Buffered_socket_operation>  _socket_operation;
    Xml_end_finder             _xml_end_finder;
    string                     _send_data;
    String_list                _received_data;
};

//-------------------------------------------------------------------------------------------------

} //namespace scheduler
} //namespace sos
